==> jhipster-core-6.0.0.tgz: package/lib/exceptions/relationship_validator.js <==
/**
 * Copyright 2013-2019 the original author or authors from the JHipster project.
 *
 * This file is part of the JHipster project, see http://www.jhipster.tech/
 * for more information.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const Validator = require('./validator');
const { exists } = require('../core/jhipster/relationship_types');

const { JPA_DERIVED_IDENTIFIER } = require('../core/jhipster/relationship_options');
const { ONE_TO_ONE, MANY_TO_MANY, MANY_TO_ONE, ONE_TO_MANY } = require('../core/jhipster/relationship_types');

class RelationshipValidator extends Validator {
  constructor() {
    super('relationship', ['from', 'to', 'type']);
  }

  validate(jdlRelationship) {
    super.validate(jdlRelationship);
    checkType(jdlRelationship);
    checkInjectedFields(jdlRelationship);
    checkForValidUseOfJPaDerivedIdentifier(jdlRelationship);
    checkForRequiredReflexiveRelationship(jdlRelationship);
    checkRelationshipType(jdlRelationship);
  }
}

module.exports = RelationshipValidator;

function checkType(jdlRelationship) {
  if (!exists(jdlRelationship.type)) {
    throw new Error(`The relationship type '${jdlRelationship.type}' doesn't exist.`);
  }
}

function checkInjectedFields(jdlRelationship) {
  if (!(jdlRelationship.injectedFieldInFrom || jdlRelationship.injectedFieldInTo)) {
    throw new Error('At least one injected field is required.');
  }
}

function checkForValidUseOfJPaDerivedIdentifier(jdlRelationship) {
  if (jdlRelationship.type !== ONE_TO_ONE && jdlRelationship.hasOption(JPA_DERIVED_IDENTIFIER)) {
    throw new Error(`Only a ${ONE_TO_ONE} relationship can have the '${JPA_DERIVED_IDENTIFIER}' option.`);
  }
}

function checkForRequiredReflexiveRelationship(jdlRelationship) {
  if (
    jdlRelationship.from.toLowerCase() === jdlRelationship.to.toLowerCase() &&
    (jdlRelationship.isInjectedFieldInFromRequired || jdlRelationship.isInjectedFieldInToRequired)
  ) {
    throw new Error(
      'Required relationships to the same entity are not supported, for relationship from and to ' +
        `'${jdlRelationship.from}'.`
    );
  }
}

function checkRelationshipType(jdlRelationship) {
  switch (jdlRelationship.type) {
    case ONE_TO_ONE:
      if (!jdlRelationship.injectedFieldInFrom) {
        throw new Error(
          `In the One-to-One relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'the source entity must possess the destination in a One-to-One ' +
            'relationship, or you must invert the direction of the relationship.'
        );
      }
      break;
    case MANY_TO_ONE:
      if (jdlRelationship.injectedFieldInFrom && jdlRelationship.injectedFieldInTo) {
        throw new Error(
          `In the Many-to-One relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'only unidirectionality is supported for a Many-to-One relationship, ' +
            'you should create a bidirectional One-to-Many relationship instead.'
        );
      }
      break;
    case MANY_TO_MANY:
      if (!jdlRelationship.injectedFieldInFrom || !jdlRelationship.injectedFieldInTo) {
        throw new Error(
          `In the Many-to-Many relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'only bidirectionality is supported for a Many-to-Many relationship.'
        );
      }
      break;
    case ONE_TO_MANY:
      return;
    default:
      // never happens, ever.
      throw new Error(`This case shouldn't have happened with type ${jdlRelationship.type}.`);
  }
}

==> jhipster-core-6.0.2.tgz: package/lib/exceptions/relationship_validator.js <==
/**
 * Copyright 2013-2019 the original author or authors from the JHipster project.
 *
 * This file is part of the JHipster project, see http://www.jhipster.tech/
 * for more information.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const Validator = require('./validator');
const { exists } = require('../core/jhipster/relationship_types');

const { JPA_DERIVED_IDENTIFIER } = require('../core/jhipster/relationship_options');
const { ONE_TO_ONE, MANY_TO_MANY, MANY_TO_ONE, ONE_TO_MANY } = require('../core/jhipster/relationship_types');

class RelationshipValidator extends Validator {
  constructor() {
    super('relationship', ['from', 'to', 'type']);
  }

  validate(jdlRelationship) {
    super.validate(jdlRelationship);
    checkType(jdlRelationship);
    checkInjectedFields(jdlRelationship);
    checkForValidUseOfJPaDerivedIdentifier(jdlRelationship);
    checkForRequiredReflexiveRelationship(jdlRelationship);
    checkRelationshipType(jdlRelationship);
  }
}

module.exports = RelationshipValidator;

function checkType(jdlRelationship) {
  if (!exists(jdlRelationship.type)) {
    throw new Error(`The relationship type '${jdlRelationship.type}' doesn't exist.`);
  }
}

function checkInjectedFields(jdlRelationship) {
  if (!(jdlRelationship.injectedFieldInFrom || jdlRelationship.injectedFieldInTo)) {
    throw new Error('At least one injected field is required.');
  }
}

function checkForValidUseOfJPaDerivedIdentifier(jdlRelationship) {
  if (jdlRelationship.type !== ONE_TO_ONE && jdlRelationship.hasOption(JPA_DERIVED_IDENTIFIER)) {
    throw new Error(`Only a ${ONE_TO_ONE} relationship can have the '${JPA_DERIVED_IDENTIFIER}' option.`);
  }
}

function checkForRequiredReflexiveRelationship(jdlRelationship) {
  if (
    jdlRelationship.from.toLowerCase() === jdlRelationship.to.toLowerCase() &&
    (jdlRelationship.isInjectedFieldInFromRequired || jdlRelationship.isInjectedFieldInToRequired)
  ) {
    throw new Error(
      'Required relationships to the same entity are not supported, for relationship from and to ' +
        `'${jdlRelationship.from}'.`
    );
  }
}

function checkRelationshipType(jdlRelationship) {
  switch (jdlRelationship.type) {
    case ONE_TO_ONE:
      if (!jdlRelationship.injectedFieldInFrom) {
        throw new Error(
          `In the One-to-One relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'the source entity must possess the source, ' +
            'or you must invert the direction of the relationship.'
        );
      }
      break;
    case MANY_TO_ONE:
      if (jdlRelationship.injectedFieldInFrom && jdlRelationship.injectedFieldInTo) {
        throw new Error(
          `In the Many-to-One relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'only unidirectionality is supported, ' +
            'you should either create a bidirectional One-to-Many relationship or ' +
            'remove the injected field in the destination entity instead.'
        );
      }
      break;
    case MANY_TO_MANY:
      if (!jdlRelationship.injectedFieldInFrom || !jdlRelationship.injectedFieldInTo) {
        throw new Error(
          `In the Many-to-Many relationship from ${jdlRelationship.from} to ${jdlRelationship.to}, ` +
            'only bidirectionality is supported.'
        );
      }
      break;
    case ONE_TO_MANY:
      return;
    default:
      // never happens, ever.
      throw new Error(`This case shouldn't have happened with type ${jdlRelationship.type}.`);
  }
}
